/*
 * K3DObject.cpp
 *
 *  Created on: Sep 28, 2008
 *      Author: Santi Ontan
 */


#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif


#ifdef _WIN32
#include "windows.h"
#endif

#include "stdlib.h"
#include "string.h"
#include "math.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "auxiliar.h"
#include "geometrics.h"
#include "2DCMC.h"
#include "Symbol.h"
#include "GLTile.h"
#include "keyboardstate.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "K3DObject.h"

#include "KObjectCreator.h"


K3DObject::K3DObject(int x, int y, int z, int sfx_volume) : KObject(x,y,sfx_volume)
{
    m_class = ktile_symbol;

	m_z = (float)z;

	m_nfaces = 0;
	m_npoints = 0;
	m_points = 0;
	m_faces = 0;
	m_normals = 0;
	m_nedges = 0;
	m_edges = 0;
	m_adjacent_faces = 0;
	m_r=1.0f;
	m_g=1.0f;
	m_b=1.0f;
	m_a=1.0f;
	m_face_colors=0;
	m_point_colors=0;
	
	collisionTile = 0;
}


K3DObject::~K3DObject()
{
	if (m_points!=0) delete []m_points;
	if (m_faces!=0) delete []m_faces;
	if (m_edges!=0) delete []m_edges;
	if (m_normals!=0) delete []m_normals;
	if (m_adjacent_faces!=0) delete []m_adjacent_faces;
	if (m_face_colors!=0) delete []m_face_colors;
	if (m_point_colors!=0) delete []m_point_colors;
	
	if (collisionTile!=0) delete collisionTile;
}

bool K3DObject::is_a(Symbol *c)
{
    if (c->cmp(k3dobject_symbol))
        return true;

    return KObject::is_a(c);
} /* KTile::is_a */


bool K3DObject::is_a(char *c)
{
    bool retval;
    Symbol *s = new Symbol(c);

    retval = is_a(s);

    delete s;

    return retval;
} /* KTile::is_a */



void K3DObject::draw(float angle,GLTManager *GLTM)
{
	draw(angle,0,0,GLTM);
} // K3DObject::draw 


void K3DObject::draw(float angle,int x_offset,int y_offset,GLTManager *GLTM)
{
	glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

	glMatrixMode(GL_MODELVIEW);

	drawInternalCartoon(x_offset,y_offset);

	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);

} // K3DObject::draw 


void K3DObject::drawInternal(int x_offset,int y_offset)
{
	float axis[3],angle;
	int i;

	glPushMatrix();
	glTranslatef(m_x+x_offset,m_y+y_offset,m_z);

	m_d.to_axis_angle(axis,&angle);
	angle*=float(180/M_PI);
	glRotatef(angle,axis[0],axis[1],axis[2]);

	glBegin(GL_TRIANGLES);
	if (m_face_colors==0 && m_point_colors==0) {
		glColor4f(m_r,m_g,m_b,m_a);
		for(i=0;i<m_nfaces;i++) {
			if (m_normals!=0) glNormal3f(m_normals[i*3],m_normals[i*3+1],m_normals[i*3+2]);
			glVertex3f(m_points[m_faces[i*3]*3],m_points[m_faces[i*3]*3+1],m_points[m_faces[i*3]*3+2]);
			glVertex3f(m_points[m_faces[i*3+1]*3],m_points[m_faces[i*3+1]*3+1],m_points[m_faces[i*3+1]*3+2]);
			glVertex3f(m_points[m_faces[i*3+2]*3],m_points[m_faces[i*3+2]*3+1],m_points[m_faces[i*3+2]*3+2]);
		} // for
	} else {
		for(i=0;i<m_nfaces;i++) {
			if (m_normals!=0) glNormal3f(m_normals[i*3],m_normals[i*3+1],m_normals[i*3+2]);
			setFacePointColor(i,m_faces[i*3]*3);
			glVertex3f(m_points[m_faces[i*3]*3],m_points[m_faces[i*3]*3+1],m_points[m_faces[i*3]*3+2]);
			setFacePointColor(i,m_faces[i*3]*3+1);
			glVertex3f(m_points[m_faces[i*3+1]*3],m_points[m_faces[i*3+1]*3+1],m_points[m_faces[i*3+1]*3+2]);
			setFacePointColor(i,m_faces[i*3]*3+2);
			glVertex3f(m_points[m_faces[i*3+2]*3],m_points[m_faces[i*3+2]*3+1],m_points[m_faces[i*3+2]*3+2]);
		} // for
	} // if
	glEnd();

	{
		K3DObject *o;
		m_parts.Rewind();
		while(m_parts.Iterate(o)) o->drawInternal(0,0);
	}
	glPopMatrix();
}


void K3DObject::drawInternalCartoon(int x_offset,int y_offset)
{
	float axis[3],angle;
	int i;

	glPushMatrix();
	glTranslatef(m_x+x_offset,m_y+y_offset,m_z);

	m_d.to_axis_angle(axis,&angle);
	angle*=float(180/M_PI);
	glRotatef(angle,axis[0],axis[1],axis[2]);

	glDepthFunc(GL_LEQUAL);
	glColor4f(0.1f,0.1f,0.1f,1.0);
	glLineWidth(1.5);
	glBegin(GL_LINES);
	for(i=0;i<m_nedges;i++) {
		glVertex3f(m_points[m_edges[i*2]*3],m_points[m_edges[i*2]*3+1],m_points[m_edges[i*2]*3+2]);
		glVertex3f(m_points[m_edges[i*2+1]*3],m_points[m_edges[i*2+1]*3+1],m_points[m_edges[i*2+1]*3+2]);
	} // for
	glEnd();

	glDepthFunc(GL_LESS);
	glColor4f(m_r,m_g,m_b,m_a);
	glBegin(GL_TRIANGLES);
	for(i=0;i<m_nfaces;i++) {
		if (m_normals!=0) glNormal3f(m_normals[i*3],m_normals[i*3+1],m_normals[i*3+2]);
		glVertex3f(m_points[m_faces[i*3]*3],m_points[m_faces[i*3]*3+1],m_points[m_faces[i*3]*3+2]);
		glVertex3f(m_points[m_faces[i*3+1]*3],m_points[m_faces[i*3+1]*3+1],m_points[m_faces[i*3+1]*3+2]);
		glVertex3f(m_points[m_faces[i*3+2]*3],m_points[m_faces[i*3+2]*3+1],m_points[m_faces[i*3+2]*3+2]);
	} // for
	glEnd();

	{
		K3DObject *o;
		m_parts.Rewind();
		while(m_parts.Iterate(o)) o->drawInternalCartoon(0,0);
	}
	glPopMatrix();
}

void K3DObject::setFacePointColor(int f,int p)
{
	if (m_point_colors!=0 && m_point_colors[p*4]>=0) {
		glColor4f(m_point_colors[p*4],m_point_colors[p*4+1],m_point_colors[p*4+2],m_point_colors[p*4+3]);
	} else {
		if (m_face_colors!=0 && m_face_colors[f*4]>=0) {
			glColor4f(m_face_colors[f*4],m_face_colors[f*4+1],m_face_colors[f*4+2],m_face_colors[f*4+3]);
		} else {
			glColor4f(m_r,m_g,m_b,m_a);
		}
	} //if
} /* K3DObject::setFacePointColor */



void K3DObject::computeNormals(void)
{
	int i;
	float v1[3],v2[3],res[3];

	m_normals = new float[m_nfaces*3];

	for(i=0;i<m_nfaces;i++) {
		v1[0]=m_points[m_faces[i*3+1]*3] - m_points[m_faces[i*3]*3];
		v1[1]=m_points[m_faces[i*3+1]*3+1] - m_points[m_faces[i*3]*3+1];
		v1[2]=m_points[m_faces[i*3+1]*3+2] - m_points[m_faces[i*3]*3+2];

		v2[0]=m_points[m_faces[i*3+2]*3] - m_points[m_faces[i*3]*3];
		v2[1]=m_points[m_faces[i*3+2]*3+1] - m_points[m_faces[i*3]*3+1];
		v2[2]=m_points[m_faces[i*3+2]*3+2] - m_points[m_faces[i*3]*3+2];

		Normal_f(v1,v2,res);

		m_normals[i*3]=res[0];
		m_normals[i*3+1]=res[1];
		m_normals[i*3+2]=res[2];
	} // for

} /* K3DObject::computeNormals */


void K3DObject::fixNormals(float x,float y, float z)
{
	int i;
	Vector v,n;
	float pe;

	for(i=0;i<m_nfaces;i++) {
		v.x=(m_points[m_faces[i*3]*3]+m_points[m_faces[i*3+1]*3]+m_points[m_faces[i*3+2]*3])/3;
		v.y=(m_points[m_faces[i*3]*3+1]+m_points[m_faces[i*3+1]*3+1]+m_points[m_faces[i*3+2]*3+1])/3;
		v.z=(m_points[m_faces[i*3]*3+2]+m_points[m_faces[i*3+1]*3+2]+m_points[m_faces[i*3+2]*3+2])/3;

		v.x=x-v.x;
		v.y=y-v.y;
		v.z=z-v.z;

		n.x = m_normals[i*3];
		n.y = m_normals[i*3+1];
		n.z = m_normals[i*3+2];

		pe = float(n*v);

		if (pe>0) {
			int tmp = m_faces[i*3+1];
			m_faces[i*3+1] = m_faces[i*3+2];
			m_faces[i*3+2] = tmp;

			m_normals[i*3] = -m_normals[i*3];
			m_normals[i*3+1] = -m_normals[i*3+1];
			m_normals[i*3+2] = -m_normals[i*3+2];
		} // if
	} // for
} /* K3DObject::fixNormals */



void K3DObject::computeEdges(float maxAngle)
{
	int i,j;
	int n_edges = 0;
	int p[9],n_common = 0;
	Vector n1,n2;
	double a;

	for(i=0;i<m_nfaces;i++) {
		for(j=i+1;j<m_nfaces;j++) {

			n_common=0;
			if (m_faces[i*3]==m_faces[j*3]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3+1]==m_faces[j*3]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+1]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+1]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+2]==m_faces[j*3]) p[n_common++]=m_faces[i*3+2];
			if (m_faces[i*3+2]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3+2];
			if (m_faces[i*3+2]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3+2];

			if (n_common>=2) {
				n1.x=m_normals[i*3];
				n1.y=m_normals[i*3+1];
				n1.z=m_normals[i*3+2];
				n2.x=m_normals[j*3];
				n2.y=m_normals[j*3+1];
				n2.z=m_normals[j*3+2];
				a = n1.angleWith(n2);
				if (a<0) a=-a;
				if (a>(M_PI/2)) a=(M_PI)-a;

				if (a>=maxAngle) {
					n_edges++;
				} // if
			} // if
		} // for
	} // for

	if (n_edges==0) return;
	m_nedges = n_edges;
	m_edges = new int[m_nedges*2];

	n_edges = 0;
	for(i=0;i<m_nfaces;i++) {
		for(j=i+1;j<m_nfaces;j++) {

			n_common=0;
			if (m_faces[i*3]==m_faces[j*3]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3];
			if (m_faces[i*3+1]==m_faces[j*3]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+1]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+1]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3+1];
			if (m_faces[i*3+2]==m_faces[j*3]) p[n_common++]=m_faces[i*3+2];
			if (m_faces[i*3+2]==m_faces[j*3+1]) p[n_common++]=m_faces[i*3+2];
			if (m_faces[i*3+2]==m_faces[j*3+2]) p[n_common++]=m_faces[i*3+2];

			if (n_common>=2) {
				n1.x=m_normals[i*3];
				n1.y=m_normals[i*3+1];
				n1.z=m_normals[i*3+2];
				n2.x=m_normals[j*3];
				n2.y=m_normals[j*3+1];
				n2.z=m_normals[j*3+2];
				a = n1.angleWith(n2);
				if (a<0) a=-a;
				if (a>(M_PI/2)) a=(M_PI)-a;

				if (a>=maxAngle) {
					m_edges[n_edges*2] = p[0];
					m_edges[n_edges*2+1] = p[1];
					n_edges++;
				} // if
			} // if
		} // for
	} // for

} /* K3DObject::computeEdges */



void K3DObject::drawShadows(int x_offset,int y_offset,float lvx,float lvy,float lvz,float depth)
{
	float axis[3],angle;
	int i,j;
	float matrix[16];
	float p[4],rp[4],rp2[4],rp3[4];
	float A,B,C,D;
	bool *visibles;
	float *rays;
	Vector light;

	if (m_adjacent_faces==0) {
		m_adjacent_faces=new int[m_nfaces*3];
		for(i=0;i<m_nfaces;i++) {
			m_adjacent_faces[i*3]=-1;
			m_adjacent_faces[i*3+1]=-1;
			m_adjacent_faces[i*3+2]=-1;
			for(j=0;j<m_nfaces;j++) {
				if (i!=j) {
					if ((m_faces[j*3]==m_faces[i*3] ||
						 m_faces[j*3+1]==m_faces[i*3] ||
						 m_faces[j*3+2]==m_faces[i*3]) &&
						(m_faces[j*3]==m_faces[i*3+1] ||
						 m_faces[j*3+1]==m_faces[i*3+1] ||
						 m_faces[j*3+2]==m_faces[i*3+1])) {
						m_adjacent_faces[i*3]=j;
					} /* if */
					if ((m_faces[j*3]==m_faces[i*3] ||
						 m_faces[j*3+1]==m_faces[i*3] ||
						 m_faces[j*3+2]==m_faces[i*3]) &&
						(m_faces[j*3]==m_faces[i*3+2] ||
						 m_faces[j*3+1]==m_faces[i*3+2] ||
						 m_faces[j*3+2]==m_faces[i*3+2])) {
						m_adjacent_faces[i*3+1]=j;
					} /* if */
					if ((m_faces[j*3]==m_faces[i*3+1] ||
						 m_faces[j*3+1]==m_faces[i*3+1] ||
						 m_faces[j*3+2]==m_faces[i*3+1]) &&
						(m_faces[j*3]==m_faces[i*3+2] ||
						 m_faces[j*3+1]==m_faces[i*3+2] ||
						 m_faces[j*3+2]==m_faces[i*3+2])) {
						m_adjacent_faces[i*3+2]=j;
					} /* if */
				} /* if */
			} /* for */
		} /* for */
	} /* if */

	visibles=new bool[m_nfaces];
	rays=new float[m_npoints*3];

	light.x=-lvx;
	light.y=-lvy;
	light.z=-lvz;
	light.normalize();
	light=light*depth;

	glPushMatrix();
	glTranslatef(m_x+x_offset,m_y+y_offset,m_z);
	m_d.to_axis_angle(axis,&angle);
	angle*=float(180/M_PI);
	glRotatef(angle,axis[0],axis[1],axis[2]);

	glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

	// We obtain the points in the propwr coordinates where they will be drawn in the world (by applying the model view matrix):
	for(i=0;i<m_npoints;i++) {
		p[0]=m_points[i*3];
		p[1]=m_points[i*3+1];
		p[2]=m_points[i*3+2];
		p[3]=1;
		ApplyMatrix(p,matrix,rp);
		rays[i*3]=rp[0];
		rays[i*3+1]=rp[1];
		rays[i*3+2]=rp[2];
	} /* for */

	// Determine which faces are visible from the point of view of the light source:
	for(i=0;i<m_nfaces;i++) {
		rp[0]=rays[m_faces[i*3]*3];
		rp[1]=rays[m_faces[i*3]*3+1];
		rp[2]=rays[m_faces[i*3]*3+2];
		rp2[0]=rays[m_faces[i*3+1]*3];
		rp2[1]=rays[m_faces[i*3+1]*3+1];
		rp2[2]=rays[m_faces[i*3+1]*3+2];
		rp3[0]=rays[m_faces[i*3+2]*3];
		rp3[1]=rays[m_faces[i*3+2]*3+1];
		rp3[2]=rays[m_faces[i*3+2]*3+2];

		A=rp[1]*(rp2[2]-rp3[2]) + rp2[1]*(rp3[2]-rp[2]) + rp3[1]*(rp[2]-rp2[2]);
		B=rp[2]*(rp2[0]-rp3[0]) + rp2[2]*(rp3[0]-rp[0]) + rp3[2]*(rp[0]-rp2[0]);
		C=rp[0]*(rp2[1]-rp3[1]) + rp2[0]*(rp3[1]-rp[1]) + rp3[0]*(rp[1]-rp2[1]);
		D=-( rp[0] *( rp2[1]*rp3[2] - rp3[1]*rp2[2] )+
			 rp2[0]*( rp3[1]*rp[2]  - rp[1]* rp3[2] )+
			 rp3[0]*( rp[1]* rp2[2] - rp2[1]*rp[2]) );

		if ((A*lvx+B*lvy+C*lvz+D)>0) visibles[i]=true;
								else visibles[i]=false;
	} /* for */

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	for(i=0;i<m_nfaces;i++) {
		if (visibles[i]) {
			rp[0]=rays[m_faces[i*3]*3];
			rp[1]=rays[m_faces[i*3]*3+1];
			rp[2]=rays[m_faces[i*3]*3+2];
			rp2[0]=rays[m_faces[i*3+1]*3];
			rp2[1]=rays[m_faces[i*3+1]*3+1];
			rp2[2]=rays[m_faces[i*3+1]*3+2];
			rp3[0]=rays[m_faces[i*3+2]*3];
			rp3[1]=rays[m_faces[i*3+2]*3+1];
			rp3[2]=rays[m_faces[i*3+2]*3+2];

			if (m_adjacent_faces[i*3]<0 ||
				!visibles[m_adjacent_faces[i*3]]) {
				glBegin(GL_TRIANGLE_STRIP);
				glVertex3f(rp[0],rp[1],rp[2]);
				light.x=rp[0]-lvx;
				light.y=rp[1]-lvy;
				light.z=rp[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(float(rp[0]+light.x),float(rp[1]+light.y),float(rp[2]+light.z));
				glVertex3f(rp2[0],rp2[1],rp2[2]);
				light.x=rp2[0]-lvx;
				light.y=rp2[1]-lvy;
				light.z=rp2[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(float(rp2[0]+light.x),float(rp2[1]+light.y),float(rp2[2]+light.z));
				glEnd();
			} // if

			if (m_adjacent_faces[i*3+1]<0 ||
				!visibles[m_adjacent_faces[i*3+1]]) {
				glBegin(GL_TRIANGLE_STRIP);
				light.x=rp3[0]-lvx;
				light.y=rp3[1]-lvy;
				light.z=rp3[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(rp3[0],rp3[1],rp3[2]);
				glVertex3f(float(rp3[0]+light.x),float(rp3[1]+light.y),float(rp3[2]+light.z));
				glVertex3f(rp[0],rp[1],rp[2]);
				light.x=rp[0]-lvx;
				light.y=rp[1]-lvy;
				light.z=rp[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(float(rp[0]+light.x),float(rp[1]+light.y),float(rp[2]+light.z));
				glEnd();
			} // if

			if (m_adjacent_faces[i*3+2]<0 ||
				!visibles[m_adjacent_faces[i*3+2]]) {
				glBegin(GL_TRIANGLE_STRIP);
				glVertex3f(rp2[0],rp2[1],rp2[2]);
				light.x=rp2[0]-lvx;
				light.y=rp2[1]-lvy;
				light.z=rp2[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(float(rp2[0]+light.x),float(rp2[1]+light.y),float(rp2[2]+light.z));
				glVertex3f(rp3[0],rp3[1],rp3[2]);
				light.x=rp3[0]-lvx;
				light.y=rp3[1]-lvy;
				light.z=rp3[2]-lvz;
				light.normalize();
				light=light*depth;
				glVertex3f(float(rp3[0]+light.x),float(rp3[1]+light.y),float(rp3[2]+light.z));
				glEnd();
			} // if
		} /* if */

	} /* for */
	glPopMatrix();

	{
		K3DObject *o;
		m_parts.Rewind();
		while(m_parts.Iterate(o)) o->drawShadows(0,0,lvx,lvy,lvz,depth);
	}

	glPopMatrix();

	delete rays;
	delete visibles;


} /* K3Object::drawShadows */


void K3DObject::createCube(int dx,int dy,int dz,float r,float g,float b,float a)
{
	m_npoints = 8;
	m_nfaces = 10;
	m_points = new float[m_npoints*3];
	m_faces = new int[m_nfaces*3];;

	m_points[0]=(float)-dx;
	m_points[1]=(float)-dy;
	m_points[2]=(float)dz;
	m_points[1*3]=(float)-dx;
	m_points[1*3+1]=(float)-dy;
	m_points[1*3+2]=(float)-dz;

	m_points[2*3]=(float)dx;
	m_points[2*3+1]=(float)-dy;
	m_points[2*3+2]=(float)dz;
	m_points[3*3]=(float)dx;
	m_points[3*3+1]=(float)-dy;
	m_points[3*3+2]=(float)-dz;

	m_points[4*3]=(float)-dx;
	m_points[4*3+1]=(float)dy;
	m_points[4*3+2]=(float)dz;
	m_points[5*3]=(float)-dx;
	m_points[5*3+1]=(float)dy;
	m_points[5*3+2]=(float)-dz;

	m_points[6*3]=(float)dx;
	m_points[6*3+1]=(float)dy;
	m_points[6*3+2]=(float)dz;
	m_points[7*3]=(float)dx;
	m_points[7*3+1]=(float)dy;
	m_points[7*3+2]=(float)-dz;

	m_faces[0] = 0;
	m_faces[1] = 1;
	m_faces[2] = 3;
	m_faces[1*3] = 3;
	m_faces[1*3+1] = 2;
	m_faces[1*3+2] = 0;

	m_faces[2*3] = 6;
	m_faces[2*3+1] = 7;
	m_faces[2*3+2] = 5;
	m_faces[3*3] = 5;
	m_faces[3*3+1] = 4;
	m_faces[3*3+2] = 6;

	m_faces[4*3] = 2;
	m_faces[4*3+1] = 3;
	m_faces[4*3+2] = 7;
	m_faces[5*3] = 7;
	m_faces[5*3+1] = 6;
	m_faces[5*3+2] = 2;

	m_faces[6*3] = 4;
	m_faces[6*3+1] = 5;
	m_faces[6*3+2] = 1;
	m_faces[7*3] = 1;
	m_faces[7*3+1] = 0;
	m_faces[7*3+2] = 4;

	m_faces[8*3] = 7;
	m_faces[8*3+1] = 3;
	m_faces[8*3+2] = 1;
	m_faces[9*3] = 5;
	m_faces[9*3+1] = 7;
	m_faces[9*3+2] = 1;

	computeNormals();
	computeEdges(3.141592f/4.0f);

	m_r = r;
	m_g = g;
	m_b = b;
	m_a = a;
}


void K3DObject::createPlane(int dx,int dy,float r,float g,float b,float a)
{
	m_npoints = 4;
	m_nfaces = 2;
	m_points = new float[m_npoints*3];
	m_faces = new int[m_nfaces*3];;

	m_points[0*3]=(float)-dx;
	m_points[0*3+1]=(float)-dy;
	m_points[0*3+2]=0;

	m_points[1*3]=(float)dx;
	m_points[1*3+1]=(float)-dy;
	m_points[1*3+2]=0;

	m_points[2*3]=(float)-dx;
	m_points[2*3+1]=(float)dy;
	m_points[2*3+2]=0;

	m_points[3*3]=(float)dx;
	m_points[3*3+1]=(float)dy;
	m_points[3*3+2]=0;

	m_faces[0*3] = 3;
	m_faces[0*3+1] = 1;
	m_faces[0*3+2] = 0;
	m_faces[1*3] = 2;
	m_faces[1*3+1] = 3;
	m_faces[1*3+2] = 0;

	computeNormals();
	computeEdges(3.141592f/4.0f);

	m_r = r;
	m_g = g;
	m_b = b;
	m_a = a;
}
