#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "stdio.h"
#include "math.h"
#include "stdlib.h"
#include "string.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "Symbol.h"
#include "2DCMC.h"
#include "auxiliar.h"
#include "GLTile.h"
#include "2DCMC.h"
#include "keyboardstate.h"
#include "randomc.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "MusicManager.h"
#include "KObject.h"
#include "Knightmare.h"
#include "KnightmareApp.h"

#include "BatAnimation.h"

// #include "font_extractor.h"


int n_lightnings = 21;
int lighning_times[21]={160,250,255,280,286,340,344,360,365,455,460,
						510,505,535,540,610,615,700,705,780,784};



int KnightmareApp::titleanimation_cycle(KEYBOARDSTATE *k)
{
	if (m_state_cycle==0) {
		if (m_bat_effect!=0) delete m_bat_effect;
		m_bat_effect = new BatEffectA(5000,1,1,1,3.0f,6.0f,200,m_GLTM);

		m_title_lighning = 0.0f;
		m_title1_alpha = 0.0f;
		m_title2_alpha = 0.0f;

		// cache some needed graphics:
		m_GLTM->get("kmlogo1");
		m_GLTM->get("kmlogo1-white");
		m_GLTM->get("kmlogo2");
		m_GLTM->get("kmlogo2-white");

		m_SFXM->SFX_play("sfx/bats", m_sfx_volume);
	} // for

	if (m_bat_effect!=0) {
		if (!m_bat_effect->cycle()) {
			delete m_bat_effect;
			m_bat_effect = 0;

		} // if
	} // if


	m_title_lighning*=0.9f;
	{
		int i,j;
		int n;
		for(i=0;i<n_lightnings;i++) {
			if (m_state_cycle==lighning_times[i]) {
				// Lighning!
				m_title_lighning = 1.0f;

				m_lightnings.Delete();
				n = 1+(rand()%2);
				for(j=0;j<n;j++) {
					float diff = 640.0f/(1+n);
					Lightning *l = new Lightning((j+1)*diff,0);
					m_lightnings.Add(l);
				} // if

				// Insert Lighning SFX here!
				if ((rand()%2)==0) m_SFXM->SFX_play("sfx/lightning1", m_sfx_volume);
							  else m_SFXM->SFX_play("sfx/lightning2", m_sfx_volume);
			} // if
		} // for
	}

	if (m_state_cycle>340) {
		if (m_title1_alpha<1.0f) m_title1_alpha+=0.01f;
	} // if

	if (m_state_cycle>510) {
		if (m_title2_alpha<1.0f) m_title2_alpha+=0.01f;
	} // if

    if (keyPress(KKEY_QUIT,-1,k) ||
    	keyPress(KKEY_FIRE,-1,k))
	{
    	return STATE_TITLE;
	}

	if (m_state_cycle>800) {
		return STATE_TITLE;
	}

	return STATE_TITLEANIMATION;
}


void KnightmareApp::titleanimation_draw(void)
{
  	glClearColor(0, 0, 0, 1);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	{
		float z = 0, y = 0;
		float a = 0;

		if (m_state_cycle<800) z = m_state_cycle*1.5f-1200.0f;
		if (m_state_cycle<800) y = 240-m_state_cycle*0.3f;
		if (m_state_cycle<800) a = (800-m_state_cycle)*0.1f;

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glTranslatef(320,140+y,z);
		glRotatef(a,0,1,0);
		if (m_state_cycle>340) {
//			m_GLTM->get("kmlogo1")->draw(1,1,1,1,0,0,0,0,1);
			m_GLTM->get("kmlogo1")->draw(1,1,1,m_title1_alpha,0,0,0,0,1);
			m_GLTM->get("kmlogo1-white")->draw(1,1,1,m_title_lighning,0,0,0,0,1);
		} // if

		if (m_state_cycle>700) {
			m_GLTM->get("kmlogo2")->draw(1,1,1,m_title2_alpha,0,0,0,0,1);
			m_GLTM->get("kmlogo2-white")->draw(1,1,1,m_title_lighning,0,0,0,0,1);
		} // if
		glPopMatrix();
	}

	{
		Lightning *l;
		m_lightnings.Rewind();
		while(m_lightnings.Iterate(l)) {
			l->draw(1,1,1,m_title_lighning);
		}
	}
	draw_square(0,0,640,480,1,1,1,m_title_lighning*m_title_lighning);

	// White bats flying away:
	if (m_bat_effect!=0) {
		m_bat_effect->draw();
	} // if

} /* KnightmareApp::titleanimation_draw */



